@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2015 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2016 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -75,6 +75,7 @@ struct interface {
 	char alias[IF_NAMESIZE];
 #endif
 	unsigned int index;
+	int active;
 	unsigned int flags;
 	sa_family_t family;
 	unsigned char hwaddr[HWADDR_LEN];
@@ -120,6 +121,8 @@ struct dhcpcd_ctx {
 	int pf_link_fd;
 #endif
 	int link_fd;
+	int seq;	/* route message sequence no */
+	int sseq;	/* successful seq no sent */
 
 #ifdef USE_SIGNALS
 	sigset_t sigset;
@@ -138,6 +141,13 @@ struct dhcpcd_ctx {
 
 	char *randomstate; /* original state */
 
+	/* Used to track the last routing message,
+	 * so we can ignore messages the parent process sent
+	 * but the child receives when forking.
+	 * getppid(2) is unreliable because we detach. */
+	pid_t ppid;	/* parent pid */
+	int pseq;	/* last seq in parent */
+
 #ifdef INET
 	struct dhcp_opt *dhcp_opts;
 	size_t dhcp_opts_len;
