@@ -25,8 +25,6 @@
  * SUCH DAMAGE.
  */
 
-#define _WITH_GETLINE /* Stop FreeBSD bitching */
-
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -145,7 +143,7 @@ const struct option cf_options[] = {
 	{"noipv4ll",        no_argument,       NULL, 'L'},
 	{"master",          no_argument,       NULL, 'M'},
 	{"renew",           no_argument,       NULL, 'N'},
-	{"nooption",        optional_argument, NULL, 'O'},
+	{"nooption",        required_argument, NULL, 'O'},
 	{"require",         required_argument, NULL, 'Q'},
 	{"static",          required_argument, NULL, 'S'},
 	{"test",            no_argument,       NULL, 'T'},
@@ -2342,8 +2340,6 @@ read_config(struct dhcpcd_ctx *ctx,
 				continue;
 			}
 			ctx->ifcc++;
-			logger(ctx, LOG_DEBUG, "allowing interface %s",
-			    ctx->ifcv[ctx->ifcc - 1]);
 			continue;
 		}
 		/* Start of an ssid block, skip if not ours */
